package riomhaire.lti.core.business;

import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.servlet.ModelAndView;
import riomhaire.lti.core.model.LTIMessage;
import riomhaire.lti.core.model.interfaces.ApplicationRegistry;
import riomhaire.lti.core.model.interfaces.Command;
import riomhaire.lti.core.model.interfaces.CommandDispatcher;

import java.util.HashMap;

/**
 * This is a simple implementation of the classic command matcher pattern where
 * we map a message to one command handler. This allows us to decouple the handler of the message from the sender.
 * We could extend this where necessary
 */
@Slf4j
@NoArgsConstructor
public class LTIMessageDispatcher implements CommandDispatcher<ModelAndView, ApplicationRegistry, LTIMessage> {
    // This stores the mapping of command to command handler
    private final HashMap<String, Command<ModelAndView, ApplicationRegistry, LTIMessage>> commands = new HashMap<>();

    // This is the action that is (optionally) executed if no match is made
    private Command<ModelAndView, ApplicationRegistry, LTIMessage> defaultCommand;

    /**
     * Adds a command handler for goven command
     *
     * @param commandName we allow same command to handle multiple requests
     * @param command     does the work
     * @return reference to dispatcher to allow fluent style usage.
     */
    @Override
    public CommandDispatcher<ModelAndView, ApplicationRegistry, LTIMessage> addCommand(String commandName, Command<ModelAndView, ApplicationRegistry, LTIMessage> command) {
        commands.put(commandName, command);
        return this;
    }


    @Override
    public ModelAndView dispatch(ApplicationRegistry registry, LTIMessage message) {
        var mav = new ModelAndView("nop");
        // If we have a handler for this command execute it
        if (message != null && commands.containsKey(message.getMessageType())) {
            mav = commands.get(message.getMessageType()).execute(registry, message);
        } else if (defaultCommand != null) {
            mav = defaultCommand.execute(registry, message);
        }
        return mav;
    }

    public void setDefaultCommand(Command<ModelAndView, ApplicationRegistry, LTIMessage> defaultCommand) {
        this.defaultCommand = defaultCommand;
    }

}
