package riomhaire.lti.core.infrastructure.facades.clientregistration;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.util.DigestUtils;
import riomhaire.lti.core.model.ClientConfiguration;
import riomhaire.lti.core.model.interfaces.ClientRegistrationResolver;

import java.util.Optional;


public class RedisBasedClientRegistrationResolver implements ClientRegistrationResolver {
    final ObjectMapper objectMapper = new ObjectMapper();
    private final RedisTemplate<String, String> template;

    public RedisBasedClientRegistrationResolver(RedisTemplate<String, String> template) {
        this.template = template;
    }

    @SneakyThrows
    @Override
    public Optional<ClientConfiguration> lookupClient(String issuer, String clientId) {
        var hash = DigestUtils.md5DigestAsHex((issuer + clientId).getBytes());
        var key = String.format("lti-client-registration-%s", hash);
        var blob = template.opsForValue().get(key);

        if (blob != null) {
            ClientConfiguration client = objectMapper.readValue(blob, ClientConfiguration.class);
            return Optional.of(client);
        }
        return Optional.empty();
    }
}
